# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models

AUDIO_MP3_PROFILE = 'audio_mp3'


def create_audio_mp3_profile(apps, schema_editor):
    """ Create audio_mp3 profile """
    Profile = apps.get_model('edxval', 'Profile')
    Profile.objects.get_or_create(profile_name=AUDIO_MP3_PROFILE)


def delete_audio_mp3_profile(apps, schema_editor):
    """ Delete audio_mp3 profile """
    Profile = apps.get_model('edxval', 'Profile')
    Profile.objects.filter(profile_name=AUDIO_MP3_PROFILE).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0010_add_video_as_foreign_key'),
    ]

    operations = [
        migrations.RunPython(create_audio_mp3_profile, delete_audio_mp3_profile),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.25 on 2019-12-05 23:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0013_thirdpartytranscriptcredentialsstate_copy_values'),
    ]

    operations = [
        migrations.AlterField(
            model_name='thirdpartytranscriptcredentialsstate',
            name='exists',
            field=models.NullBooleanField(default=False, help_text='Transcript credentials state'),
        ),
    ]



"""
Util methods to be used in api and models.
"""

from __future__ import absolute_import

import json

import six
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.files.storage import get_storage_class
from fs.path import combine
from pysrt import SubRipFile
from rest_framework import status
from rest_framework.response import Response


class TranscriptFormat:
    """Tuple representing transcriptformat choices."""
    SRT = 'srt'
    SJSON = 'sjson'

    CHOICES = (
        (SRT, 'SubRip'),
        (SJSON, 'SRT JSON')
    )


# 3rd Party Transcription Plans
THIRD_PARTY_TRANSCRIPTION_PLANS = {

    'Cielo24': {
        'display_name': 'Cielo24',
        'turnaround': {
            'PRIORITY': 'Priority (24 hours)',
            'STANDARD': 'Standard (48 hours)'
        },
        'fidelity': {
            'MECHANICAL': {
                'display_name': 'Mechanical (75% accuracy)',
                'languages': {
                    'nl': 'Dutch',
                    'en': 'English',
                    'fr': 'French',
                    'de': 'German',
                    'it': 'Italian',
                    'es': 'Spanish',
                }
            },
            'PREMIUM': {
                'display_name': 'Premium (95% accuracy)',
                'languages': {
                    'en': 'English',
                }
            },
            'PROFESSIONAL': {
                'display_name': 'Professional (99% accuracy)',
                'languages': {
                    'ar': 'Arabic',
                    'zh-tw': 'Chinese - Mandarin (Traditional)',
                    'zh-cmn': 'Chinese - Mandarin (Simplified)',
                    'zh-yue': 'Chinese - Cantonese (Traditional)',
                    'nl': 'Dutch',
                    'en': 'English',
                    'fr': 'French',
                    'de': 'German',
                    'he': 'Hebrew',
                    'hi': 'Hindi',
                    'it': 'Italian',
                    'ja': 'Japanese',
                    'ko': 'Korean',
                    'pt': 'Portuguese',
                    'ru': 'Russian',
                    'es': 'Spanish',
                    'tr': 'Turkish',
                }
            },
        }
    },
    '3PlayMedia': {
        'display_name': '3Play Media',
        'turnaround': {
            'two_hour': '2 hours',
            'same_day': 'Same day',
            'rush': '24 hours (rush)',
            'expedited': '2 days (expedited)',
            'standard': '4 days (standard)',
            'extended': '10 days (extended)'
        },
        'languages': {
            'en': 'English',
            'fr': 'French',
            'de': 'German',
            'it': 'Italian',
            'nl': 'Dutch',
            'es': 'Spanish',
            'el': 'Greek',
            'pt': 'Portuguese',
            'zh': 'Chinese',
            'ar': 'Arabic',
            'he': 'Hebrew',
            'ru': 'Russian',
            'ja': 'Japanese',
            'sv': 'Swedish',
            'cs': 'Czech',
            'da': 'Danish',
            'fi': 'Finnish',
            'id': 'Indonesian',
            'ko': 'Korean',
            'no': 'Norwegian',
            'pl': 'Polish',
            'th': 'Thai',
            'tr': 'Turkish',
            'vi': 'Vietnamese',
            'ro': 'Romanian',
            'hu': 'Hungarian',
            'ms': 'Malay',
            'bg': 'Bulgarian',
            'tl': 'Tagalog',
            'sr': 'Serbian',
            'sk': 'Slovak',
            'uk': 'Ukrainian',
        },
        # Valid translations -- a mapping of source languages to the
        # translatable target languages.
        'translations': {
            'es': [
                'en'
            ],
            'en': [
                'el', 'en', 'zh', 'vi',
                'it', 'ar', 'cs', 'id',
                'es', 'ru', 'nl', 'pt',
                'no', 'tr', 'tl', 'th',
                'ro', 'pl', 'fr', 'bg',
                'uk', 'de', 'da', 'fi',
                'hu', 'ja', 'he', 'sr',
                'ko', 'sv', 'sk', 'ms'
            ],
        }
    }
}


def video_image_path(video_image_instance, filename):  # pylint:disable=unused-argument
    """
    Returns video image path.

    Arguments:
        video_image_instance (VideoImage): This is passed automatically by models.CustomizableImageField
        filename (str): name of image file
    """
    return u'{}{}'.format(settings.VIDEO_IMAGE_SETTINGS.get('DIRECTORY_PREFIX', ''), filename)


def get_video_image_storage():
    """
    Return the configured django storage backend.
    """
    if hasattr(settings, 'VIDEO_IMAGE_SETTINGS'):
        return get_storage_class(
            settings.VIDEO_IMAGE_SETTINGS.get('STORAGE_CLASS'),
        )(**settings.VIDEO_IMAGE_SETTINGS.get('STORAGE_KWARGS', {}))

    # during edx-platform loading this method gets called but settings are not ready yet
    # so in that case we will return default(FileSystemStorage) storage class instance
    return get_storage_class()()


def video_transcript_path(video_transcript_instance, filename):  # pylint:disable=unused-argument
    """
    Returns video transcript path.

    Arguments:
        video_transcript_instance (VideoTranscript): This is passed automatically by models.CustomizableFileField
        filename (str): name of image file
    """
    return u'{}{}'.format(settings.VIDEO_TRANSCRIPTS_SETTINGS.get('DIRECTORY_PREFIX', ''), filename)


def get_video_transcript_storage():
    """
    Return the configured django storage backend for video transcripts.
    """
    if hasattr(settings, 'VIDEO_TRANSCRIPTS_SETTINGS'):
        return get_storage_class(
            settings.VIDEO_TRANSCRIPTS_SETTINGS.get('STORAGE_CLASS'),
        )(**settings.VIDEO_TRANSCRIPTS_SETTINGS.get('STORAGE_KWARGS', {}))

    # during edx-platform loading this method gets called but settings are not ready yet
    # so in that case we will return default(FileSystemStorage) storage class instance
    return get_storage_class()()


def create_file_in_fs(file_data, file_name, file_system, static_dir):
    """
    Writes file in specific file system.

    Arguments:
        file_data (str): Data to store into the file.
        file_name (str): File name of the file to be created.
        file_system (OSFS): Import file system.
        static_dir (str): The Directory to retrieve transcript file.
    """
    with file_system.open(combine(static_dir, file_name), 'wb') as f:
        f.write(file_data.encode('utf-8'))


def get_transcript_format(transcript_content):
    """
    Returns transcript format.

    Arguments:
        transcript_content (str): Transcript file content.
    """
    try:
        json.loads(transcript_content)
    except ValueError:
        # With error handling (set to 'ERROR_RAISE'), we will be getting
        # the exception if something went wrong in parsing the transcript.

        srt_subs = SubRipFile.from_string(transcript_content, error_handling=SubRipFile.ERROR_RAISE)
        if srt_subs:
            return TranscriptFormat.SRT
    return TranscriptFormat.SJSON


def validate_generated_images(value, max_items):
    """
    Validate data before saving to database.

    Arguments:
        value(list): list to be validated
        max_items (int): maximum number of items in a list

    Returns:
        list if validation is successful

    Raises:
        ValidationError
    """
    if len(value) > max_items:
        raise ValidationError(
            u'list must not contain more than {max_items} items.'.format(max_items=max_items)
        )

    if all(isinstance(item, six.string_types) for item in value) is False:
        raise ValidationError(u'list must only contain strings.')

    return value


def invalidate_fernet_cached_properties(model, fields):
    """
    Invalidates transcript credential fernet field's cached properties.

    Arguments:
        model (class): Model class containing fernet fields.
        fields (list):  A list of fernet fields whose cache is to be invalidated.
    """
    for field_name in fields:
        try:
            field = model._meta.get_field(field_name)
            del field.keys
            del field.fernet_keys
            del field.fernet
        except AttributeError:
            pass


def validate_request_params(data, attributes):
    """
    Checks if the given set of attributes are missing from request and returns a response if true.

    Arguments:
        data(dict): params dict
        attributes(list): list of required attributes
    Returns:
        HTTP Response if params are missing,else None
    """
    missing = [attr for attr in attributes if not data[attr]]
    if missing:
        return Response(
            status=status.HTTP_400_BAD_REQUEST,
            data={'message': '{missing} must be specified.'.format(missing=' and '.join(missing))}

        )
    return None



"""
A module containing transcripts utils.
"""
# pylint: disable=inconsistent-return-statements

from __future__ import absolute_import

import json
import logging

import requests
import six
from django.conf import settings
from pysrt import SubRipFile, SubRipItem, SubRipTime
from pysrt.srtexc import Error
from six import text_type
from six.moves import range

from edxval.enum import TranscriptionProviderErrorType
from edxval.exceptions import TranscriptsGenerationException
from edxval.models import TranscriptProviderType

LOGGER = logging.getLogger(__name__)


class Transcript:
    """
    Container for transcript methods.
    """
    SRT = 'srt'
    SJSON = 'sjson'

    @staticmethod
    def generate_sjson_from_srt(srt_subs):
        """
        Generate transcripts from sjson to SubRip (*.srt).

        Arguments:
            srt_subs(SubRip): "SRT" subs object

        Returns:
            Subs converted to "SJSON" format.
        """
        sub_starts = []
        sub_ends = []
        sub_texts = []
        for sub in srt_subs:
            sub_starts.append(sub.start.ordinal)
            sub_ends.append(sub.end.ordinal)
            sub_texts.append(sub.text.replace('\n', ' '))

        sjson_subs = {
            'start': sub_starts,
            'end': sub_ends,
            'text': sub_texts
        }
        return sjson_subs

    @staticmethod
    def generate_srt_from_sjson(sjson_subs):
        """
        Generate transcripts from sjson to SubRip (*.srt)

        Arguments:
            sjson_subs (dict): `sjson` subs.

        Returns:
            Subtitles in SRT format.
        """
        output = ''

        equal_len = len(sjson_subs['start']) == len(sjson_subs['end']) == len(sjson_subs['text'])
        if not equal_len:
            return output

        for i in range(len(sjson_subs['start'])):
            item = SubRipItem(
                index=i,
                start=SubRipTime(milliseconds=sjson_subs['start'][i]),
                end=SubRipTime(milliseconds=sjson_subs['end'][i]),
                text=sjson_subs['text'][i]
            )
            output += (six.text_type(item))
            output += '\n'
        return output

    @classmethod
    def convert(cls, content, input_format, output_format):
        """
        Convert transcript `content` from `input_format` to `output_format`.

        Arguments:
            content: Transcript content byte-stream.
            input_format: Input transcript format.
            output_format: Output transcript format.

        Accepted input formats: sjson, srt.
        Accepted output format: srt, sjson.

        Raises:
            TranscriptsGenerationException: On parsing the invalid srt
            content during conversion from srt to sjson.
        """
        assert input_format in ('srt', 'sjson')
        assert output_format in ('srt', 'sjson')

        # Decode the content with utf-8-sig which will also
        # skip byte order mark(BOM) character if found.
        content = content.decode('utf-8-sig')

        if input_format == output_format:
            return content

        if input_format == 'srt':

            if output_format == 'sjson':
                try:
                    # With error handling (set to 'ERROR_RAISE'), we will be getting
                    # the exception if something went wrong in parsing the transcript.
                    srt_subs = SubRipFile.from_string(content, error_handling=SubRipFile.ERROR_RAISE)
                except Error as ex:  # Base exception from pysrt
                    raise TranscriptsGenerationException(text_type(ex))

                return json.dumps(cls.generate_sjson_from_srt(srt_subs))

        if input_format == 'sjson':

            if output_format == 'srt':
                return cls.generate_srt_from_sjson(json.loads(content))


def get_cielo_token_response(username, api_secure_key):
    """
    Returns Cielo24 api token.

    Arguments:
        username(str): Cielo24 username
        api_securekey(str): Cielo24 api key

    Returns:
        Response : Http response object
    """
    cielo_api_url = settings.CIELO24_SETTINGS.get('CIELO24_LOGIN_URL', "https://sandbox.cielo24.com/api/account/login")
    return requests.get(cielo_api_url, params={
            'v': settings.CIELO24_SETTINGS.get('CIELO24_API_VERSION', 1),
            'username': username,
            'securekey': api_secure_key
        })


def get_api_token(username, api_key):
    """
    Returns api token if valid credentials are provided.
    """
    response = get_cielo_token_response(username=username, api_secure_key=api_key)
    if not response.ok:
        api_token = None
        LOGGER.warning(
            '[Transcript Credentials] Unable to get api token --  response %s --  status %s.',
            response.text,
            response.status_code,
        )
    else:
        api_token = json.loads(response.content.decode('utf-8'))['ApiToken']

    return api_token


def validate_missing_attributes(provider, attributes, credentials):
    """
    Returns error message if provided attributes are not presents in credentials.
    """
    error_message, error_type = None, None

    missing = [attr for attr in attributes if attr not in credentials]
    if missing:
        error_message = '{missing} must be specified for {provider}.'.format(
            provider=provider,
            missing=' and '.join(missing)
        )
        error_type = TranscriptionProviderErrorType.MISSING_REQUIRED_ATTRIBUTES

    return error_type, error_message


def validate_transcript_credentials(provider, **credentials):
    """
    Validates transcript credentials.

     Validations:
     Providers must be either 3PlayMedia or Cielo24.
     In case of:
            3PlayMedia - 'api_key' and 'api_secret_key' are required.
            Cielo24 - Valid 'api_key' and 'username' are required.
    """
    error_type, error_message, validated_credentials = None, '', {}
    if provider in [TranscriptProviderType.CIELO24, TranscriptProviderType.THREE_PLAY_MEDIA]:
        if provider == TranscriptProviderType.CIELO24:
            must_have_props = ('org', 'api_key', 'username')
            error_type, error_message = validate_missing_attributes(provider, must_have_props, credentials)

            if not error_message:
                # Get cielo api token and store it in api_key.
                api_token = get_api_token(credentials['username'], credentials['api_key'])
                if api_token:
                    validated_credentials.update({
                        'org': credentials['org'],
                        'api_key': api_token
                    })
                else:
                    error_message = 'Invalid credentials supplied.'
                    error_type = TranscriptionProviderErrorType.INVALID_CREDENTIALS
        else:
            must_have_props = ('org', 'api_key', 'api_secret_key')
            error_type, error_message = validate_missing_attributes(provider, must_have_props, credentials)
            if not error_message:
                validated_credentials.update({
                    'org': credentials['org'],
                    'api_key': credentials['api_key'],
                    'api_secret': credentials['api_secret_key']
                })
    else:
        error_message = 'Invalid provider {provider}.'.format(provider=provider)
        error_type = TranscriptionProviderErrorType.INVALID_PROVIDER

    return error_type, error_message, validated_credentials



# -*- coding: utf-8 -*-
"""
Tests for transcript utils.
"""
from __future__ import absolute_import

import json
import textwrap

import responses
from ddt import data, ddt, unpack
from django.test import TestCase
from mock import patch
from rest_framework import status

from edxval.enum import TranscriptionProviderErrorType
from edxval.exceptions import TranscriptsGenerationException
from edxval.models import TranscriptProviderType
from edxval.transcript_utils import Transcript, validate_transcript_credentials


@ddt
class TestTranscriptUtils(TestCase):
    """
    Tests transcripts conversion util.
    """
    def setUp(self):
        super(TestTranscriptUtils, self).setUp()

        self.srt_transcript = textwrap.dedent(u"""\
            0
            00:00:10,500 --> 00:00:13,000
            Elephant&#39;s Dream 大象的梦想

            1
            00:00:15,000 --> 00:00:18,000
            At the left we can see...

        """).encode('utf8')

        self.sjson_transcript = textwrap.dedent(u"""\
            {
                "start": [
                    10500,
                    15000
                ],
                "end": [
                    13000,
                    18000
                ],
                "text": [
                    "Elephant&#39;s Dream 大象的梦想",
                    "At the left we can see..."
                ]
            }
        """).encode('utf8')

    @data(
        ('invalid_input_format', 'sjson'),
        ('sjson', 'invalid_output_format'),
        ('invalid_input_format', 'invalid_output_format')
    )
    @unpack
    def test_invalid_transcript_format(self, input_format, output_format):
        """
        Tests that transcript conversion raises `AssertionError` on invalid input/output formats.
        """
        with self.assertRaises(AssertionError):
            Transcript.convert(self.sjson_transcript, input_format, output_format)

    def test_convert_srt_to_srt(self):
        """
        Tests that srt to srt conversion works as expected.
        """
        expected = self.srt_transcript.decode('utf-8')
        actual = Transcript.convert(self.srt_transcript, 'srt', 'srt')
        self.assertEqual(actual, expected)

    def test_convert_sjson_to_srt(self):
        """
        Tests that the sjson transcript is successfully converted into srt format.
        """
        expected = self.srt_transcript.decode('utf-8')
        actual = Transcript.convert(self.sjson_transcript, 'sjson', 'srt')
        self.assertEqual(actual, expected)

    def test_convert_srt_to_sjson(self):
        """
        Tests that the srt transcript is successfully converted into sjson format.
        """
        expected = self.sjson_transcript.decode('utf-8')
        actual = Transcript.convert(self.srt_transcript, 'srt', 'sjson')
        self.assertDictEqual(json.loads(actual), json.loads(expected))

    def test_convert_invalid_srt_to_sjson(self):
        """
        Tests that TranscriptsGenerationException was raises on trying
        to convert invalid srt transcript to sjson.
        """
        invalid_srt_transcript = b'invalid SubRip file content'
        with self.assertRaises(TranscriptsGenerationException):
            Transcript.convert(invalid_srt_transcript, 'srt', 'sjson')


@ddt
class TestCredentialsUtils(TestCase):
    """
    Test Suite for various transcript credential utilities.
    """

    CIELO24_LOGIN_URL = "https://sandbox.cielo24.com/api/account/login"

    @patch('edxval.transcript_utils.LOGGER')
    @responses.activate
    def test_cielo24_error(self, mock_logger):
        """
        Test that when invalid cielo credentials are supplied, we get correct error response.
        """
        expected_error_message = 'Invalid credentials supplied.'
        responses.add(
            responses.GET,
            self.CIELO24_LOGIN_URL,
            body=json.dumps({'error': expected_error_message}),
            status=status.HTTP_400_BAD_REQUEST
        )

        credentials = {
            'org': 'test',
            'provider': TranscriptProviderType.CIELO24,
            'api_key': 'test-api-key',
            'username': 'test-cielo-user',
            'api_secret_key': ''
        }
        error_type, error_message, _ = validate_transcript_credentials(**credentials)
        self.assertEqual(error_type, TranscriptionProviderErrorType.INVALID_CREDENTIALS)
        self.assertEqual(error_message, expected_error_message)

        mock_logger.warning.assert_called_with(
            '[Transcript Credentials] Unable to get api token --  response %s --  status %s.',
            json.dumps({'error': error_message}),
            status.HTTP_400_BAD_REQUEST
        )

    @data(
        {
            'provider': 'unsupported-provider'
        },
        {
            'org': 'test',
            'api_key': 'test-api-key'
        }
    )
    def test_transcript_credentials_invalid_provider(self, credentials):
        """
        Test that validating credentials gives proper error in case of invalid provider.
        """
        provider = credentials.pop('provider', '')
        error_type, error_message, _ = validate_transcript_credentials(provider, **credentials)
        self.assertEqual(error_type, TranscriptionProviderErrorType.INVALID_PROVIDER)
        self.assertEqual(error_message, 'Invalid provider {provider}.'.format(provider=provider))

    @data(
        (
                {'provider': TranscriptProviderType.CIELO24},
                'org and api_key and username'
        ),
        (
                {'provider': TranscriptProviderType.THREE_PLAY_MEDIA},
                'org and api_key and api_secret_key'
        ),
        (
                {'provider': TranscriptProviderType.CIELO24, 'org': 'test-org'},
                'api_key and username'
        ),
        (
                {'provider': TranscriptProviderType.CIELO24, 'org': 'test-org', 'api_key': 'test-api-key'},
                'username'
        ),
        (
                {'org': 'test', 'provider': TranscriptProviderType.THREE_PLAY_MEDIA, 'api_key': 'test-api-key'},
                'api_secret_key'
        )
    )
    @unpack
    def test_transcript_credentials_error(self, credentials, missing_keys):
        """
        Test that validating credentials gives proper error in case of invalid input.
        """
        provider = credentials.pop('provider')
        expected_error_message = '{missing} must be specified for {provider}.'.format(
            provider=provider,
            missing=missing_keys
        )
        error_type, error_message, _ = validate_transcript_credentials(provider, **credentials)
        self.assertEqual(error_type, TranscriptionProviderErrorType.MISSING_REQUIRED_ATTRIBUTES)
        self.assertEqual(error_message, expected_error_message)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.25 on 2019-12-06 21:18
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0014_transcript_credentials_state_retype_exists'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='thirdpartytranscriptcredentialsstate',
            name='exists',
        ),
    ]



"""
Admin Module Test Cases
"""
from __future__ import absolute_import

from django.contrib.admin.sites import AdminSite
from django.contrib.messages.storage.fallback import FallbackStorage
from django.http import HttpRequest
from django.test import TestCase

from edxval.admin import ProfileAdmin
from edxval.models import Profile


class AdminTestCase(TestCase):
    """
    Test Case module for Profile Admin
    """
    def setUp(self):
        super(AdminTestCase, self).setUp()
        self.request = HttpRequest()
        self.conf_admin = ProfileAdmin(Profile, AdminSite())
        self.request.session = 'session'
        self.request._messages = FallbackStorage(self.request)  # pylint: disable=protected-access

    def test_default_fields(self):
        """
        Test: checking fields
        """
        self.assertEqual(
            list(self.conf_admin.get_form(self.request).base_fields),
            ['profile_name']
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.25 on 2019-12-03 05:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0011_data__add_audio_mp3_profile'),
    ]

    operations = [
        migrations.AddField(
            model_name='thirdpartytranscriptcredentialsstate',
            name='has_creds',
            field=models.BooleanField(default=False, help_text='Transcript credentials state'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def copy_column_values(apps, schema_editor):
    """
    Copy the 'exists' field's value into the 'has_creds' field.
    """
    ThirdPartyTranscriptCredentialsState = apps.get_model('edxval', 'ThirdPartyTranscriptCredentialsState')
    for credentials_state in ThirdPartyTranscriptCredentialsState.objects.all():
        credentials_state.has_creds = credentials_state.exists
        credentials_state.save()


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0012_thirdpartytranscriptcredentialsstate_has_creds'),
    ]

    operations = [
        migrations.RunPython(
            copy_column_values,
            reverse_code=migrations.RunPython.noop,  # Allow reverse migrations, but make it a no-op.
        ),
    ]



# -*- coding: utf-8 -*-
""" Test for models """
from __future__ import absolute_import

from cryptography.fernet import InvalidToken
from django.test import TestCase, override_settings

from edxval.models import TranscriptCredentials, TranscriptProviderType, Video, VideoTranscript
from edxval.tests import constants
from edxval.utils import invalidate_fernet_cached_properties


class VideoTranscriptTest(TestCase):
    """
    Test VideoTranscript model
    """

    def setUp(self):
        """
        Creates Profile objects and a video object
        """
        self.transcript_data = constants.VIDEO_TRANSCRIPT_CIELO24
        super(VideoTranscriptTest, self).setUp()

    def test_filename_property_new_line(self):
        """
        Test that filename does not contain any "\n".
        New line is not permmited in response headers.
        """
        video = Video.objects.create(**constants.VIDEO_DICT_NEW_LINE)
        video_trancript = VideoTranscript.objects.create(
            video=video,
            language_code=self.transcript_data['language_code'],
            file_format=self.transcript_data['file_format'],
            provider=self.transcript_data['provider'],
        )

        self.assertNotIn('\n', video_trancript.filename)
        assert str(video_trancript) == "en Transcript for new-line-not-allowed"


class TranscriptCredentialsTest(TestCase):
    """
    Test suite for TranscriptCredentials model.
    """
    def setUp(self):
        super(TranscriptCredentialsTest, self).setUp()
        self.credentials_data = {
            'org': 'MAx',
            'provider': TranscriptProviderType.CIELO24,
            'api_key': 'test-key',
            'api_secret': 'test-secret'
        }
        TranscriptCredentials.objects.create(**self.credentials_data)

    def test_decryption(self):
        """
        Verify that with unchanged encryption key, the decrypted data will be same as before encryption.
        """
        credentials = TranscriptCredentials.objects.get(
            org=self.credentials_data['org'], provider=self.credentials_data['provider']
        )
        self.assertEqual(credentials.api_key, self.credentials_data['api_key'])
        self.assertEqual(credentials.api_secret, self.credentials_data['api_secret'])

    def test_decryption_with_new_key(self):
        """
        Verify that with a new encryption key, previously encrypted data cannot be decrypted.
        """
        invalidate_fernet_cached_properties(TranscriptCredentials, ['api_key', 'api_secret'])
        with override_settings(FERNET_KEYS=['new-key']):
            with self.assertRaises(InvalidToken):
                TranscriptCredentials.objects.get(
                    org=self.credentials_data['org'], provider=self.credentials_data['provider']
                )



"""
Url file for django app edxval.
"""

from __future__ import absolute_import

from django.conf import settings
from django.conf.urls import url

from edxval import views

urlpatterns = [
    url(r'^videos/$', views.VideoList.as_view(), name='video-list'),
    url(
        r'^videos/(?P<edx_video_id>[-\w]+)$',
        views.VideoDetail.as_view(),
        name='video-detail'
    ),
    url(
        r'^videos/status/$',
        views.VideoStatusView.as_view(),
        name='video-status-update'
    ),
    url(
        r'^videos/missing-hls/$',
        views.HLSMissingVideoView.as_view(),
        name='hls-missing-video'
    ),
    url(
        r'^videos/video-transcripts/create/$',
        views.VideoTranscriptView.as_view(),
        name='create-video-transcript'
    ),
    url(
        r'^videos/video-images/update/$',
        views.VideoImagesView.as_view(),
        name='update-video-images'
    ),
    url(
        r'^videos/transcript-credentials/(?P<provider>[\w]*)/(?P<org>[\w]*)$',
        views.TranscriptCredentialsView.as_view(),
        name='transcript-credentials'
    ),
    url(
        r'^videos/transcript-preferences/{}$'.format(settings.COURSE_ID_PATTERN),
        views.TranscriptPreferenceView.as_view(),
        name='transcript-preferences'
    )
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-03-25 07:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import fernet_fields.fields
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0015_remove_thirdpartytranscriptcredentialsstate_exists'),
    ]

    operations = [
        migrations.CreateModel(
            name='TranscriptCredentials',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('org', models.CharField(help_text='This value must match the value of organization in studio/edx-platform.', max_length=50, verbose_name='Organization')),
                ('provider', models.CharField(choices=[('Custom', 'Custom'), ('3PlayMedia', '3PlayMedia'), ('Cielo24', 'Cielo24')], max_length=50, verbose_name='Transcript provider')),
                ('api_key', fernet_fields.fields.EncryptedTextField(max_length=255, verbose_name='API key')),
                ('api_secret', fernet_fields.fields.EncryptedTextField(max_length=255, verbose_name='API secret')),
            ],
            options={
                'verbose_name_plural': 'Transcript Credentials',
            },
        ),
        migrations.AlterUniqueTogether(
            name='transcriptcredentials',
            unique_together=set([('org', 'provider')]),
        ),
    ]



"""
Serializers for Video Abstraction Layer

Serialization is usually sent through the VideoSerializer which uses the
EncodedVideoSerializer which uses the profile_name as it's profile field.
"""
from __future__ import absolute_import

from rest_framework import serializers
from rest_framework.fields import DateTimeField, IntegerField

from edxval.models import CourseVideo, EncodedVideo, Profile, TranscriptPreference, Video, VideoImage, VideoTranscript


class EncodedVideoSerializer(serializers.ModelSerializer):
    """
    Serializer for EncodedVideo object.

    Uses the profile_name as it's profile value instead of a Profile object.
    """
    profile = serializers.SlugRelatedField(
        slug_field="profile_name",
        queryset=Profile.objects.all()
    )

    # Django Rest Framework v3 doesn't enforce minimum values for
    # PositiveIntegerFields, so we need to specify the min value explicitly.
    bitrate = IntegerField(min_value=0)
    file_size = IntegerField(min_value=0)

    # Django Rest Framework v3 converts datetimes to unicode by default.
    # Specify format=None to leave them as datetimes.
    created = DateTimeField(required=False, format=None)
    modified = DateTimeField(required=False, format=None)

    class Meta:
        model = EncodedVideo
        fields = (
            "created",
            "modified",
            "url",
            "file_size",
            "bitrate",
            "profile",
        )

    def get_identity(self, data):
        """
        This hook is required for bulk update.
        We need to override the default, to use the slug as the identity.
        """
        return data.get('profile', None)


class TranscriptSerializer(serializers.ModelSerializer):
    """
    Serializer for VideoTranscript objects
    """
    class Meta:
        model = VideoTranscript
        fields = ('video_id', 'url', 'language_code', 'provider', 'file_format')

    video_id = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()

    def get_video_id(self, transcript):
        """
        Returns an edx video ID for the related video.
        """
        return transcript.video.edx_video_id

    def get_url(self, transcript):
        """
        Retrieves the transcript url.
        """
        return transcript.url()

    def validate(self, data):  # pylint: disable=arguments-differ
        """
        Validates the transcript data.
        """
        video_id = self.context.get('video_id')
        video = Video.get_or_none(edx_video_id=video_id)
        if not video:
            raise serializers.ValidationError('Video "{video_id}" is not valid.'.format(video_id=video_id))

        data.update(video=video)
        return data

    def create(self, validated_data):
        """
        Create the video transcript.
        """
        return VideoTranscript.create(**validated_data)


class CourseSerializer(serializers.RelatedField):
    """
    Field for CourseVideo
    """
    def to_representation(self, course_video):  # pylint: disable=arguments-differ
        """
        Returns a serializable representation of a CourseVideo instance.
        """
        return {
            course_video.course_id: course_video.image_url()
        }

    def to_internal_value(self, data):
        """
        Convert data into CourseVideo instance and image filename tuple.
        """
        course_id = data
        course_video = image = ''
        if data:
            if isinstance(data, dict):
                (course_id, image), = list(data.items())

            course_video = CourseVideo(course_id=course_id)
            course_video.full_clean(exclude=['video'])

        return course_video, image


class VideoSerializer(serializers.ModelSerializer):
    """
    Serializer for Video object

    encoded_videos takes a list of dicts EncodedVideo data.
    """
    encoded_videos = EncodedVideoSerializer(many=True)
    courses = CourseSerializer(
        many=True,
        read_only=False,
        required=False,
        queryset=CourseVideo.objects.all()
    )
    url = serializers.SerializerMethodField()

    # Django Rest Framework v3 converts datetimes to unicode by default.
    # Specify format=None to leave them as datetimes.
    created = DateTimeField(required=False, format=None)

    class Meta:
        model = Video
        lookup_field = "edx_video_id"
        exclude = ('id',)

    def get_url(self, obj):
        """
        Return relative url for the object
        """
        return obj.get_absolute_url()

    def validate(self, data):  # pylint: disable=arguments-differ
        """
        Check that the video data is valid.
        """
        if data is not None and not isinstance(data, dict):
            raise serializers.ValidationError("Invalid data")

        try:
            profiles = [ev["profile"] for ev in data.get("encoded_videos", [])]
            if len(profiles) != len(set(profiles)):
                raise serializers.ValidationError("Invalid data: duplicate profiles")
        except KeyError:
            raise serializers.ValidationError("profile required for deserializing")
        except TypeError:
            raise serializers.ValidationError("profile field needs to be a profile_name (str)")

        # Clean course_video list from any invalid data.
        course_videos = [(course_video, image) for course_video, image in data.get('courses', []) if course_video]
        data['courses'] = course_videos

        return data

    def create(self, validated_data):
        """
        Create the video and its nested resources.
        """
        courses = validated_data.pop("courses", [])
        encoded_videos = validated_data.pop("encoded_videos", [])

        video = Video.objects.create(**validated_data)

        EncodedVideo.objects.bulk_create(
            EncodedVideo(video=video, **video_data)
            for video_data in encoded_videos
        )

        # The CourseSerializer will already have converted the course data
        # to CourseVideo models, so we can just set the video and save.
        # Also create VideoImage objects if an image filename is present
        for course_video, image_name in courses:
            course_video.video = video
            course_video.save()
            if image_name:
                VideoImage.create_or_update(course_video, image_name)

        return video

    def update(self, instance, validated_data):
        """
        Update an existing video resource.
        """
        instance.status = validated_data["status"]
        instance.client_video_id = validated_data["client_video_id"]
        instance.duration = validated_data["duration"]
        instance.save()

        # Set encoded videos
        instance.encoded_videos.all().delete()
        EncodedVideo.objects.bulk_create(
            EncodedVideo(video=instance, **video_data)
            for video_data in validated_data.get("encoded_videos", [])
        )

        # Set courses
        # NOTE: for backwards compatibility with the DRF v2 behavior,
        # we do NOT delete existing course videos during the update.
        # Also update VideoImage objects if an image filename is present
        for course_video, image_name in validated_data.get("courses", []):
            course_video.video = instance
            course_video.save()
            if image_name:
                VideoImage.create_or_update(course_video, image_name)

        return instance


class TranscriptPreferenceSerializer(serializers.ModelSerializer):
    """
    Serializer for TranscriptPreference
    """

    class Meta:
        model = TranscriptPreference
        fields = (
            'course_id',
            'provider',
            'cielo24_fidelity',
            'cielo24_turnaround',
            'three_play_turnaround',
            'preferred_languages',
            'video_source_language',
            'modified',
        )

    preferred_languages = serializers.SerializerMethodField()

    def get_preferred_languages(self, transcript_preference):
        """
        Returns python list for preferred_languages model field.
        """
        return transcript_preference.preferred_languages



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import django.utils.timezone
import edxval.models
import model_utils.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0005_videoimage'),
    ]

    operations = [
        migrations.CreateModel(
            name='TranscriptPreference',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('course_id', models.CharField(unique=True, max_length=255, verbose_name=u'Course ID')),
                ('provider', models.CharField(max_length=20, verbose_name=u'Provider', choices=[(u'Custom', u'Custom'), (u'3PlayMedia', u'3PlayMedia'), (u'Cielo24', u'Cielo24')])),
                ('cielo24_fidelity', models.CharField(blank=True, max_length=20, null=True, verbose_name=u'Cielo24 Fidelity', choices=[(u'MECHANICAL', u'Mechanical, 75% Accuracy'), (u'PREMIUM', u'Premium, 95% Accuracy'), (u'PROFESSIONAL', u'Professional, 99% Accuracy')])),
                ('cielo24_turnaround', models.CharField(blank=True, max_length=20, null=True, verbose_name=u'Cielo24 Turnaround', choices=[(u'STANDARD', u'Standard, 48h'), (u'PRIORITY', u'Priority, 24h')])),
                ('three_play_turnaround', models.CharField(blank=True, max_length=20, null=True, verbose_name=u'3PlayMedia Turnaround', choices=[(u'extended_service', u'10-Day/Extended'), (u'default', u'4-Day/Default'), (u'expedited_service', u'2-Day/Expedited'), (u'rush_service', u'24 hour/Rush'), (u'same_day_service', u'Same Day')])),
                ('preferred_languages', edxval.models.ListField(default=[], verbose_name=u'Preferred Languages', max_items=50, blank=True)),
                ('video_source_language', models.CharField(help_text=u'This specifies the speech language of a Video.', max_length=50, null=True, verbose_name=u'Video Source Language', blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VideoTranscript',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('video_id', models.CharField(help_text=u'It can be an edx_video_id or an external video id', max_length=255)),
                ('transcript', edxval.models.CustomizableFileField(null=True, blank=True)),
                ('language_code', models.CharField(max_length=50, db_index=True)),
                ('provider', models.CharField(default=u'Custom', max_length=30, choices=[(u'Custom', u'Custom'), (u'3PlayMedia', u'3PlayMedia'), (u'Cielo24', u'Cielo24')])),
                ('file_format', models.CharField(db_index=True, max_length=20, choices=[(u'srt', u'SubRip'), (u'sjson', u'SRT JSON')])),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='videotranscript',
            unique_together=set([('video_id', 'language_code')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0002_data__default_profiles'),
    ]

    operations = [
        migrations.AddField(
            model_name='coursevideo',
            name='is_hidden',
            field=models.BooleanField(default=False, help_text=u'Hide video for course.'),
        ),
    ]



"""
Module containing all the Enumerations used in the API.
"""


class TranscriptionProviderErrorType:
    """
    Transcription provider's errors enumeration.
    """
    INVALID_CREDENTIALS = 1
    INVALID_PROVIDER = 2
    MISSING_REQUIRED_ATTRIBUTES = 3



"""
Admin file for django app edxval.
"""
from __future__ import absolute_import

from django.contrib import admin

from .models import (
    CourseVideo,
    EncodedVideo,
    Profile,
    ThirdPartyTranscriptCredentialsState,
    TranscriptCredentials,
    TranscriptPreference,
    Video,
    VideoImage,
    VideoTranscript,
)


class ProfileAdmin(admin.ModelAdmin):
    """ Admin for profile """
    list_display = ('id', 'profile_name')
    list_display_links = ('id', 'profile_name')
    admin_order_field = 'profile_name'


class EncodedVideoInline(admin.TabularInline):
    """ Admin for EncodedVideo """
    model = EncodedVideo


class CourseVideoInline(admin.TabularInline):
    """ Admin for CourseVideo """
    model = CourseVideo
    extra = 0
    verbose_name = "Course"
    verbose_name_plural = "Courses"


class VideoAdmin(admin.ModelAdmin):
    """ Admin for Video """
    list_display = (
        'id', 'edx_video_id', 'client_video_id', 'duration', 'created', 'status'
    )
    list_display_links = ('id', 'edx_video_id')
    search_fields = ('id', 'edx_video_id', 'client_video_id', 'status')
    list_per_page = 50
    admin_order_field = 'edx_video_id'
    inlines = [CourseVideoInline, EncodedVideoInline]


class VideoImageAdmin(admin.ModelAdmin):
    """ Admin for VideoImage """
    raw_id_fields = ('course_video', )
    list_display = ('get_course_video', 'image', 'generated_images')
    search_fields = ('id', 'course_video__course_id', 'course_video__video__edx_video_id', 'generated_images')

    def get_course_video(self, obj):
        """ get course video """
        return u'"{course_id}" -- "{edx_video_id}" '.format(
            course_id=obj.course_video.course_id,
            edx_video_id=obj.course_video.video.edx_video_id
        )

    get_course_video.admin_order_field = 'course_video'
    get_course_video.short_description = 'Course Video'

    model = VideoImage

    verbose_name = 'Video Image'
    verbose_name_plural = 'Video Images'


class CourseVideoAdmin(admin.ModelAdmin):
    """ Admin for CourseVideo """
    list_display = ('course_id', 'get_video_id', 'is_hidden')
    search_fields = ('id', 'course_id', 'video__status', 'video__edx_video_id')

    def get_video_id(self, obj):
        """ get video id """
        return obj.video.edx_video_id

    get_video_id.admin_order_field = 'video'
    get_video_id.short_description = 'edX Video Id'

    model = CourseVideo
    verbose_name = 'Course Video'
    verbose_name_plural = 'Course Videos'


class VideoTranscriptAdmin(admin.ModelAdmin):
    """ Admin for VideoTranscript """
    raw_id_fields = ('video',)
    list_display = ('get_video', 'language_code', 'provider', 'file_format')
    search_fields = ('id', 'video__edx_video_id', 'language_code')

    def get_video(self, transcript):
        """ get video """
        return transcript.video.edx_video_id if getattr(transcript, 'video', False) else ''

    get_video.admin_order_field = 'video'
    get_video.short_description = 'Video'

    model = VideoTranscript
    verbose_name = 'Video Transcript'
    verbose_name_plural = 'Video Transcripts'


class TranscriptPreferenceAdmin(admin.ModelAdmin):
    """ Admin for TranscriptPreference """
    list_display = ('course_id', 'provider', 'video_source_language', 'preferred_languages')

    model = TranscriptPreference


class TranscriptCredentialsAdmin(admin.ModelAdmin):
    """
    Admin for TranscriptCredentials model.
    """
    model = TranscriptCredentials
    exclude = ('api_key', 'api_secret')


class ThirdPartyTranscriptCredentialsStateAdmin(admin.ModelAdmin):
    """ Admin for ThirdPartyTranscriptCredentialsState  """
    list_display = ('org', 'provider', 'has_creds', 'created', 'modified')

    model = ThirdPartyTranscriptCredentialsState
    verbose_name = 'Organization Transcript Credential State'
    verbose_name_plural = 'Organization Transcript Credentials State'


admin.site.register(Profile, ProfileAdmin)
admin.site.register(Video, VideoAdmin)
admin.site.register(VideoTranscript, VideoTranscriptAdmin)
admin.site.register(TranscriptCredentials, TranscriptCredentialsAdmin)
admin.site.register(TranscriptPreference, TranscriptPreferenceAdmin)
admin.site.register(VideoImage, VideoImageAdmin)
admin.site.register(CourseVideo, CourseVideoAdmin)
admin.site.register(ThirdPartyTranscriptCredentialsState, ThirdPartyTranscriptCredentialsStateAdmin)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import django.utils.timezone
import edxval.models
import model_utils.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0004_data__add_hls_profile'),
    ]

    operations = [
        migrations.CreateModel(
            name='VideoImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('image', edxval.models.CustomizableImageField(null=True, blank=True)),
                ('generated_images', edxval.models.ListField()),
                ('course_video', models.OneToOneField(related_name='video_image', to='edxval.CourseVideo', on_delete = models.CASCADE)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models

HLS_PROFILE = 'hls'


def create_hls_profile(apps, schema_editor):
    """ Create hls profile """
    Profile = apps.get_model("edxval", "Profile")
    Profile.objects.get_or_create(profile_name=HLS_PROFILE)


def delete_hls_profile(apps, schema_editor):
    """ Delete hls profile """
    Profile = apps.get_model("edxval", "Profile")
    Profile.objects.filter(profile_name=HLS_PROFILE).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0003_coursevideo_is_hidden'),
    ]

    operations = [
        migrations.RunPython(create_hls_profile, delete_hls_profile),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models

DEFAULT_PROFILES = [
    "desktop_mp4",
    "desktop_webm",
    "mobile_high",
    "mobile_low",
    "youtube",
]


def create_default_profiles(apps, schema_editor):
    """ Add default profiles """
    Profile = apps.get_model("edxval", "Profile")
    for profile in DEFAULT_PROFILES:
        Profile.objects.get_or_create(profile_name=profile)


def delete_default_profiles(apps, schema_editor):
    """ Remove default profiles """
    Profile = apps.get_model("edxval", "Profile")
    Profile.objects.filter(profile_name__in=DEFAULT_PROFILES).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('edxval', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(create_default_profiles, delete_default_profiles),
    ]
